@@ -50,8 +50,7 @@ setClass("triangularMatrix",
 ## Virtual class of numeric matrices
 setClass("dMatrix",
 	 representation(x = "numeric", "VIRTUAL"), contains = "Matrix",
-	 validity = function(object)
-	 .Call(dMatrix_validate, object))
+	 validity = function(object) .Call(dMatrix_validate, object))
 
 ## Virtual class of integer matrices
 setClass("iMatrix",
@@ -123,13 +122,13 @@ setClass("TsparseMatrix", representation(i = "integer", j = "integer", "VIRTUAL"
 
 setClass("CsparseMatrix", representation(i = "integer", p = "integer", "VIRTUAL"),
 	 contains = "sparseMatrix",
-	 prototype = prototype(p = 0:0),# to be valid
+	 prototype = prototype(p = 0L),# to be valid
          validity = function(object) .Call(Csparse_validate, object)
          )
 
 setClass("RsparseMatrix", representation(p = "integer", j = "integer", "VIRTUAL"),
 	 contains = "sparseMatrix",
-	 prototype = prototype(p = 0:0),# to be valid
+	 prototype = prototype(p = 0L),# to be valid
 	 validity = function(object) .Call(Rsparse_validate, object)
          )
 
@@ -174,9 +173,7 @@ setClass("dgeMatrix", contains = c("ddenseMatrix", "generalMatrix"),
 ## numeric, dense, non-packed, triangular matrices
 setClass("dtrMatrix",
 	 contains = c("ddenseMatrix", "triangularMatrix"),
-	 validity =
-	 function(object) .Call(dtrMatrix_validate, object)
-	 )
+	 validity = function(object) .Call(dense_nonpacked_validate, object))
 
 ## numeric, dense, packed, triangular matrices
 setClass("dtpMatrix",
@@ -188,9 +185,7 @@ setClass("dtpMatrix",
 ## numeric, dense, non-packed symmetric matrices
 setClass("dsyMatrix",
          contains = c("ddenseMatrix", "symmetricMatrix"),
-	 validity =
-	 function(object) .Call(dsyMatrix_validate, object)
-	 )
+	 validity = function(object) .Call(dense_nonpacked_validate, object))
 
 ## numeric, dense, packed symmetric matrices
 setClass("dspMatrix",
@@ -213,13 +208,14 @@ setClass("dppMatrix", contains = "dspMatrix",
 
 ## logical, dense, general matrices
 setClass("lgeMatrix", contains = c("ldenseMatrix", "generalMatrix"),
+         ## since "lge" inherits from "ldenseMatrix", only need this:
 	 ## checks that length( @ x) == prod( @ Dim):
-	 validity = function(object) stopifnot(length(object@x) == prod(object@Dim))
-	 )
+	 validity = function(object) .Call(dense_nonpacked_validate, object))
 ## i.e. "lgeMatrix" cannot be packed, but "ldenseMatrix" can ..
 
 ## logical, dense, non-packed, triangular matrices
 setClass("ltrMatrix",
+	 validity = function(object) .Call(dense_nonpacked_validate, object),
 	 contains = c("ldenseMatrix", "triangularMatrix"))
 
 ## logical, dense, packed, triangular matrices
@@ -228,26 +224,26 @@ setClass("ltpMatrix",
 
 ## logical, dense, non-packed symmetric matrices
 setClass("lsyMatrix",
+	 validity = function(object) .Call(dense_nonpacked_validate, object),
 	 contains = c("ldenseMatrix", "symmetricMatrix"))
 
 ## logical, dense, packed symmetric matrices
 setClass("lspMatrix",
 	 contains = c("ldenseMatrix", "symmetricMatrix"),
 	 validity = function(object) .Call(dspMatrix_validate, object)
-	 ## "dsp" and "lsp" have the same validate
+	 ## "dsp", "lsp" and "nsp" have the same validate
 	 )
 
 ##----- nonzero pattern dense Matrices -- "for completeness"
 
 ## logical, dense, general matrices
 setClass("ngeMatrix", contains = c("ndenseMatrix", "generalMatrix"),
-	 ## checks that length( @ x) == prod( @ Dim):
-	 validity = function(object)
-         stopifnot(length(object@x) == prod(object@Dim)))
+	 validity = function(object) .Call(dense_nonpacked_validate, object))
 ## i.e. "ngeMatrix" cannot be packed, but "ndenseMatrix" can ..
 
 ## logical, dense, non-packed, triangular matrices
 setClass("ntrMatrix",
+	 validity = function(object) .Call(dense_nonpacked_validate, object),
 	 contains = c("ndenseMatrix", "triangularMatrix"))
 
 ## logical, dense, packed, triangular matrices
@@ -256,20 +252,21 @@ setClass("ntpMatrix",
 
 ## logical, dense, non-packed symmetric matrices
 setClass("nsyMatrix",
+	 validity = function(object) .Call(dense_nonpacked_validate, object),
 	 contains = c("ndenseMatrix", "symmetricMatrix"))
 
 ## logical, dense, packed symmetric matrices
 setClass("nspMatrix",
 	 contains = c("ndenseMatrix", "symmetricMatrix"),
 	 validity = function(object) .Call(dspMatrix_validate, object)
-	 ## "dsp" and "nsp" have the same validate
+	 ## "dsp", "lsp" and "nsp" have the same validate
 	 )
 
 
 ## 'diagonalMatrix' already has validity checking
-## diagonal, numeric matrices;	    "d*" has 'x' slot :
+## diagonal, numeric matrices; "dMatrix" has 'x' slot :
 setClass("ddiMatrix", contains = c("diagonalMatrix", "dMatrix"))# or "dMatrix"
-## diagonal, logical matrices; "ldense*" has 'x' slot :
+## diagonal, logical matrices; "lMatrix" has 'x' slot :
 setClass("ldiMatrix", contains = c("diagonalMatrix", "lMatrix"))
 
 setClass("corMatrix", representation(sd = "numeric"), contains = "dpoMatrix",
@@ -278,6 +275,8 @@ setClass("corMatrix", representation(sd = "numeric"), contains = "dpoMatrix",
 	     n <- object@Dim[2]
 	     if(length(sd <- object@sd) != n)
 		 return("'sd' slot must be of length 'dim(.)[1]'")
+	     if(any(!is.finite(sd)))# including NA
+		 return("'sd' slot has non-finite entries")
 	     if(any(sd < 0))
 		 return("'sd' slot has negative entries")
 	     TRUE
@@ -536,10 +535,23 @@ setClass("isRMatrix",
 	 )
 }##--not yet--
 
-##-------------------- permutation ----------------------------------------
+##-------------------- index and permutation matrices--------------------------
+
+setClass("indMatrix", representation(perm = "integer"),
+	 contains = c("sparseMatrix", "generalMatrix"),
+	 validity = function(object) {
+	     n <-  object@Dim[1]
+	     d <-  object@Dim[2]
+	     perm <- object@perm
+	     if (length(perm) != n)
+		 return(paste("length of 'perm' slot must be", n))
+	     if(n > 0 && (any(perm > d) | any(perm < 1)))
+		 return("'perm' slot is not a valid index")
+	     TRUE
+	 })
 
 setClass("pMatrix", representation(perm = "integer"),
-	 contains = c("sparseMatrix", "generalMatrix"),
+	 contains = c("indMatrix"),
 	 validity = function(object) {
 	     d <- object@Dim
 	     if (d[2] != (n <- d[1])) return("pMatrix must be square")
@@ -620,16 +632,17 @@ setClass("denseLU", contains = "LU",
 	 validity = function(object) .Call(LU_validate, object))
 
 setClass("sparseLU", contains = "LU",
-	 representation(L = "dgCMatrix", U = "dgCMatrix",
+	 representation(L = "dtCMatrix", U = "dtCMatrix",
 			p = "integer", q = "integer"))
 
 ##--- QR ---
 
 setClass("sparseQR", contains = "MatrixFactorization",
 	 representation(V = "dgCMatrix", beta = "numeric",
-			p = "integer", R = "dgCMatrix", q = "integer"))
+			p = "integer", R = "dgCMatrix", q = "integer"),
+	 validity = function(object) .Call(sparseQR_validate, object))
 
-##-- "SPQR" ---> ./spqr.R  for noew
+##-- "SPQR" ---> ./spqr.R  for now
 
 ## "denseQR" -- ?  (``a version of''  S3 class "qr")
 
@@ -671,6 +684,18 @@ setClass("Schur", contains = "MatrixFactorization",
 
 ### Class Union :  no inheritance, but is(*, <class>) :
 
+setClassUnion("mMatrix", members = c("matrix", "Matrix"))
+
+## CARE: Sometimes we'd want all those for which 'x' contains all the data.
+##       e.g. Diagonal() is "ddiMatrix" with 'x' slot of length 0, does *not* contain 1
+setClassUnion("xMatrix", ## those Matrix classes with an 'x' slot
+              c("dMatrix",
+                "iMatrix",
+                "lMatrix",
+                "ndenseMatrix",
+                "zMatrix"))
+
+
 ## Definition  Packed := dense with length( . @x) < prod( . @Dim)
 ##	       ~~~~~~
 ## REPLACED the following with	isPacked() in ./Auxiliaries.R :
@@ -686,7 +711,7 @@ setClass("Schur", contains = "MatrixFactorization",
 ## e.g. for "Arith" methods:
 setClassUnion("numLike", members = c("numeric", "logical"))
 
-setClassUnion("numIndex", members = "numeric")
+##setClassUnion("numIndex", members = "numeric")
 
 ## Note "rle" is a sealed oldClass (and "virtual" as w/o prototype)
 setClass("rleDiff", representation(first = "numLike", rle = "rle"),
@@ -779,11 +804,12 @@ setClass("sparseVector",
          ## note that "numeric" contains "integer" (if I like it or not..)
 	 prototype = prototype(length = 0),
          validity = function(object) {
-             n <- object@length
-	     if(any(!is.finite(i <- object@i)))# no NA's !
-		 "'i' slot is not all finite"
-	     else if(any(i < 1) || any(i > n))
-		 sprintf("'i' must be in 1:%d", n)
+	     n <- object@length
+	     if(anyNA(i <- object@i))	 "'i' slot has NAs"
+	     else if(any(!is.finite(i))) "'i' slot is not all finite"
+	     else if(any(i < 1))	 "'i' must be >= 1"
+	     else if(n == 0 && length(i))"'i' must be empty when the object length is zero"
+	     else if(any(i > n)) sprintf("'i' must be in 1:%d", n)
 	     else if(is.unsorted(i, strictly=TRUE))
 		 "'i' must be sorted strictly increasingly"
              else TRUE
@@ -792,6 +818,8 @@ setClass("sparseVector",
 ##' initialization -- ensuring that  'i' is sorted (and 'x' alongside)
 setMethod("initialize", "sparseVector", function(.Object, i, x, ...)
       {
+	  ## NB: could simplify for R(-devel) >= 2015-01-14 (i.e. >= 3.2.0),
+	  ##	 assigning 'i' and 'x' and calling  callNextMethod() {w/o arg.s}
 	  has.x <- !missing(x)
 	  if(!missing(i)) {
 	      .Object@i <- ## (be careful to assign in all cases)
@@ -807,7 +835,7 @@ setMethod("initialize", "sparseVector", function(.Object, i, x, ...)
 		  else i
 	  }
 	  if(has.x) .Object@x <- x
-	  callNextMethod()
+	  callNextMethod(.Object, ...)
       })
 
 .validXspVec <- function(object) {
